/* 
 * My File System Interface
 * 
 * test-myremove2
 *
 * Tests my file system interface:
 * - creates an writes simultaneously data to files 
 * - removes a file
 * - check the number of existing files
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 4
#define F_SIZE (2*512)
#define CHUNK 256

int main(int argc, char **argv)
{
   int num, fd;
   char* buf;
   int numberOfFiles1,numberOfFiles2;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];
   
   // mandatory to init the myfs layer
   my_init_lib();
   
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/f-%d",f);
   }

   /*
    * create and write simultaneously data to files
    */

   // create and open all files
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to all files in chunks of 256 bytes

   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close all files
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // listdir  root directory
   if(my_listdir("/",&buf, &numberOfFiles1)<0){
      printf("[test] error listing directory.\n");
      return -1;
   }

   // removes  file 'f-2' from the root directory
   fd = my_remove("/f-2");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }

  // check files in root directory
  // listdir  root directory
   if(my_listdir("/",&buf, &numberOfFiles2)<0){
      printf("[test] error listing directory.\n");
      return -1;
   }
   
  // check files in root directory
  if ((numberOfFiles1==(numberOfFiles2+1))&&(numberOfFiles1==N_FILES))
	printf("[test] PASSED.\n");
    
   return 0;
}


