/* 
 * My File System Interface
 * 
 * test-myremove3
 *
 * Tests my file system interface:
 * - creates and writes 4 files simultaneously ( 2 in root directory and 2 in a root subdirectory)
 * - removes 1 file in root directory plus another in a root subdirectory
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 4
#define F_SIZE (3*512)
#define CHUNK 512

int main(int argc, char **argv)
{
   int num, fd;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];
   
   
   // mandatory to init the myfs layer
   my_init_lib();

   // create directory tree
   char* subdir[] = {
      "/a",
   };

   //Creating a root subdirectory
   if (my_mkdir(subdir[0]) < 0) {
      printf("[test] failed: unable to create directory '%s'.\n",subdir[0]);
      return -1;
   }
      
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],( f < (N_FILES/2) )?("/a/f-%d"):("/f-%d"),f);
   }


   /*
    * create and write simultaneously data to files
    */

   // create and open all files
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to all files in chunks of 512 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close all files
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }


   // removes  file '/a/f-0' 
   fd = my_remove("/a/f-0");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }
   
   // removes  file 'f-2' from root directory
   fd = my_remove("/f-2");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }
   
   // Dump file system occupied data blocks
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
   
      
   printf("\n[test] remove was successfull if:\n	- blocks 13, 17 and 21 are allocated to file '/a/f-1'.\n	- blocks 15, 19 and 23 are allocated to file '/f-3'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");
      
   return 0;
}

