/* 
 * My File System Interface
 * 
 * test-myremove4
 *
 * Tests my file system interface:
 * - creates and writes files simultaneously and
 * - checks the copy-on-write after removing a file sharing data blocks
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 2
#define F_SIZE (2*512)
#define CHUNK 256

int main(int argc, char **argv)
{
   int num, fd;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];
   
   
   // mandatory to init the myfs layer
   my_init_lib();

   // create directory tree
   char* subdir[] = {
      "/a",
   };

   //Creating a root subdirectory
   if (my_mkdir(subdir[0]) < 0) {
      printf("[test] failed: unable to create directory '%s'.\n",subdir[0]);
      return -1;
   }
      
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/a/f-%d",f);
   }


   /*
    * create and write simultaneously data to files
    */

   // create and open all files
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to all files in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close all files
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // copy  file '/a/f-0' to '/f10' 
   fd = my_copy("/a/f-0", "/f-10");
   if (fd < 0) {
      printf("[test] unable to copy a file.\n");
      return -1;
   }
   
   // copy  file '/a/f-0' to '/f20' 
   fd = my_copy("/a/f-0", "/f-20");
   if (fd < 0) {
      printf("[test] unable to copy a file.\n");
      return -1;
   }
   
   // remove  file '/a/f-0' 
   fd = my_remove("/a/f-0");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }

   // Dump file system occupied data blocks
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
   
   printf("\n[test] remove was successfull if:\n	- blocks 12 and 14 are allocated to files '/f-10' and '/f-20'.\n	- blocks 13 and 15 are allocated to file '/a/f-1'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");   

   return 0;
}


