/* 
 * My File System Interface
 * 
 * test-mycopy2
 *
 * Tests my file system interface:
 * - Copying a  file
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 1
#define F_SIZE (2*512)
#define CHUNK 256

int main(int argc, char **argv)
{
   int num, fd;
   char* buf;
   int numberOfFiles;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];
      
   // mandatory to init the myfs layer
   my_init_lib();
   
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/f-%d",f);
   }

   /*
    * create and write data to file
    */

   // create and open all files
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to the file in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close the file
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // copy  file 'f-0' to 'f10' in the root directory
   fd = my_copy("/f-0", "/f-10");
   if (fd < 0) {
      printf("[test] unable to copy a file.\n");
      return -1;
   }

  // check the number of files in root directory
   if(my_listdir("/",&buf, &numberOfFiles)<0){
      printf("[test] error listing directory.\n");
      return -1;
   }
 
   if (numberOfFiles==(N_FILES +1)) {
   
	   fd = my_open("/f-10",0);
	   if (fd < 0) {
	      printf("[test] unable to open file.\n");
	      return -1;
	   }   
	   // read and compare file contents with written data
	   char buffer[512];
	   int read = 0;
	   while (1) {
	      num = my_read(fd,buffer,sizeof(buffer));
	      if (num < 0) {
		 printf("[test] error reading data from file.\n");
		 return -1;
	      }
	      if (num == 0) {
		 break;
	      }
	      if (strncmp(&fdata[0][read],buffer,num) != 0) {
		 printf("[test] file contents differ from data.\n");
		 return -1;
	      }
	      read += num;
	   }
	     
	   // close the file
	   if (my_close(fd) < 0) {
	      printf("[test] error closing file.\n");
	      return -1;
	   }

   
	  printf("[test] PASSED.\n");
   }
   
    return 0;
}


