/* 
 * My File System Interface
 * 
 * test-mycopy3
 *
 * Tests my file system interface:
 * - creates and writes a file and
 *   checks the integrity of the written data
 * - copies file 'f-0' with 2 data blocks to file 'f-10' and
 * - checks the integrity of the file 'f-10'
 * - Checks whether copy-on-write is implemented  
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 1
#define F_SIZE (2*512)
#define CHUNK 256

int main(int argc, char **argv)
{
   int num, fd;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];
      
   // mandatory to init the myfs layer
   my_init_lib();
   
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/f-%d",f);
   }

   /*
    * create and write data to a file
    */

   // create and open the file
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to the file in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close the file
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // copy  file '/f-0' to '/f10' in the root directory
   fd = my_copy("/f-0", "/f-10");
   if (fd < 0) {
      printf("[test] unable to copy a file.\n");
      return -1;
   }

  // Dump file system occupied data blocks
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
  
   printf("\n[test] copy-on-write was successfull if: \n	-  blocks 11 and 12 are both allocated to files '/f-0' and '/f-10'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");   

    return 0;
}


