/* 
 * My File System Interface
 * 
 * test-mycopy4
 *
 * Tests the my file system interface:
 * - Creates 2 different files in root directory
 * - Copies the 1st file to 3 new files 
 * - Checks copied files integrity
 * - Checks whether copy-on-write is implemented 
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 5
#define F_SIZE (2*512)
#define CHUNK 256

int main(int argc, char **argv)
{
   int num, fd;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   char buffer[CHUNK];
   int ffd[N_FILES];

      
   // mandatory to init the myfs layer
   my_init_lib();
   
   // generate files names with the same data
   for (int f = 0; f < N_FILES; f++) {
     if (f < (N_FILES/2)) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
	 } 
      sprintf(fname[f],"/f-%d",f);
   }
   
   // create and open the file
   for (int f = 0; f <(N_FILES/2); f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to the file in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < (N_FILES/2); f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }
 
   // close the file
   for (int f = 0; f < (N_FILES/2); f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }
 
   // copy  file 'f-0' to 'f-2', 'f-3', and 'f-4' in the root directory
   for(int f = (N_FILES/2); f < N_FILES; f++) {	   
	   fd = my_copy("/f-0", fname[f]);
	   if (fd < 0) {
		  printf("[test] unable to copy a file.\n");
		  return -1;
	   }

	   // check that the file exists by reopening it without the O_CREATE flag
	   fd = my_open(fname[f],0);
	   if (fd < 0) {
		  printf("[test] unable to open existing.\n");
		  return -1;
	   }
	   
	   // test the read data in chunks of 256 bytes
	   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
			 num = my_read(fd, buffer, CHUNK);
			 if (num < CHUNK) {
				printf("[test] error reading data from file.\n");
				return -1;
			 }
			 if (memcmp(&fdata[0][ptr],buffer,num) != 0) {
				printf("[test] file contents differ from data.\n");
				return -1;
			 }
	   }
	   
	 
	   if (my_close(fd) < 0) {
		  printf("[test] error closing file.\n");
		  return -1;
	   }
	}
   // Dump file system occupied data blocks
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
   
   printf("\n[test] copy-on-write was successfull if:\n	- blocks 11 and 13 are allocated to files '/f-0', '/f-2', 'f/-3' and '/f-4'.\n    	- blocks 12 and 14 are allocated to file '/f-1'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");   

   return 0;
}


