/* 
 * My File System Interface
 * 
 * test-append2
 *
 * Tests the my file system interface:
 * - creates and writes 2 files simultaneously (file sizes are not block size multiples)
 * - appends the 2nd file to the first one
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define F_SIZE1 520
#define F_SIZE2 600
#define CHUNK 512

int main(int argc, char **argv)
{
   int num, fd;  
   
   // mandatory to init the myfs layer
   my_init_lib();   
   
   // create the empty file 'file1.txt' in the root directory
   fd = my_open("/file1.txt",O_CREATE);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // fill in a buffer with ascii chars
   char data1[F_SIZE1];
   for (int i = 0; i < sizeof(data1); data1[i]='a'+i%10, i++);

   // write the data to the file
   num = my_write(fd, data1, sizeof(data1));
   if (num < sizeof(data1)) {
      printf("[test] error writing data to file.\n");
      return -1;
   }
   
   // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }
   
   // create the empty file 'file2.txt' in the root directory
   fd = my_open("/file2.txt",O_CREATE);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // fill in a buffer with ascii chars
   char data2[F_SIZE2];
   for (int i = 0; i < sizeof(data2); data2[i]='B'+i%10, i++);

   // write the data to the file
   num = my_write(fd, data2, sizeof(data2));
   if (num < sizeof(data2)) {
      printf("[test] error writing data to file.\n");
      return -1;
   }
   
   // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }

   // append  file '/file2.txt' to '/file1.txt' in the root directory
   fd = my_append("/file1.txt", "/file2.txt");
   if (fd < 0) {
      printf("[test] unable to append a file.\n");
      return -1;
   }
   
 
   // reopen file 'file1.txt'
   fd = my_open("/file1.txt",0);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }
 
   
   // read file contents to buffer
   char buffer[3*512];
   int ptr = 0;
   num = 0;
   do {
      num = my_read(fd,&buffer[ptr],CHUNK);
      if (num < 0) {
         printf("[test] error reading from file.\n");
         return -1;
      }
      ptr+=num;
   } while(num > 0);
   
    // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }
  

   // test if file sizes differ
   if (ptr != (F_SIZE1+F_SIZE2)) {
      printf("[test] file size differ from the expected value. \n");
      return -1;
   }
   
   printf("[test] PASSED.\n");
      
   return 0;
}


