/* 
 * My File System Interface
 * 
 * test-append4
 *
 * Tests the my file system interface:
 * - creates and writes 2 files simultaneously (file sizes are block size multiples) and
 * - appends one file to another  
 * - Checks the allocated blocks to both files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 2
#define F_SIZE (2*512)
#define CHUNK 512

int main(int argc, char **argv)
{
   int num, fd;   
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];   

   // mandatory to init the myfs layer
   my_init_lib();
   
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/f-%d",f);
   }

   /*
    * create and write simultaneously data to files
    */

   // create and open all files
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to all files in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close all files
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // append  file '/f-1' to '/f-0' in the root directory
   fd = my_append("/f-0", "/f-1");
   if (fd < 0) {
      printf("[test] unable to append a file.\n");
      return -1;
   }
   
   // Dump file system occupied data blocks
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
   
   
   printf("\n[test] append was successfull if:\n	EITHER:\n	- blocks 11 and 13 are allocated to file '/f-0'.\n	- blocks 12 and 14 are allocated to files '/f-0' and '/f-1'.\n	OR:\n	- blocks 11, 13, 15 and 16 are allocated to file '/f-0'.\n	- blocks 12 and 14 are allocated to files '/f-1'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");

   return 0;
}


