 /* 
 * My File System Interface
 * 
 * test-mydefrag4
 *
 * Tests my file system interface:
 * - creates and writes data to 4 files simultaneously
 * - removes 2 files
 * - creates a new file by copying 
 * - appends the other file to the new one
 * - defragments the file system 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>
#include "tfuncs.h"
#define N_FILES 3
#define F_SIZE (3*512)
#define CHUNK 256
#define D_SIZE (10*512)


int main(int argc, char **argv)
{

   int fd, num;
   char fdata[N_FILES][F_SIZE];
   char fname[N_FILES][16];
   int ffd[N_FILES];

   // mandatory to init the myfs layer
   my_init_lib();
   
   // create the empty file 'file1.txt' in the root directory
   fd = my_open("/file1.txt",O_CREATE);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // fill in a buffer with ascii chars
   char data[D_SIZE];
   for (int i = 0; i < sizeof(data); data[i]='a'+i%10, i++);

   // write the data to the file
   num = my_write(fd, data, sizeof(data));
   if (num < sizeof(data)) {
      printf("[test] error writing data to file.\n");
      return -1;
   }
   
   // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }
   
   
   // generate files names and random ascii data
   for (int f = 0; f < N_FILES; f++) {
      gen_data_rand_ascii(fdata[f],F_SIZE);
      fdata[f][F_SIZE-1] = '\0';
      sprintf(fname[f],"/f-%d",f);
   }

   /*
    * create and write data to the files
    */

   // create and open the file
   for (int f = 0; f < N_FILES; f++) {
      ffd[f] = my_open(fname[f],O_CREATE);
      if (ffd[f] < 0) {
         printf("[test] unable to open file.\n");
         return -1;
      }
   }

   // write the random data to the files in chunks of 256 bytes
   for (int ptr = 0; ptr < F_SIZE; ptr += CHUNK) {
      for (int f = 0; f < N_FILES; f++) {
         num = my_write(ffd[f], &fdata[f][ptr], CHUNK);
         if (num < CHUNK) {
            printf("[test] error writing data to file.\n");
            return -1;
         }
       }
   }

   // close the files
   for (int f = 0; f < N_FILES; f++) {
      if (my_close(ffd[f]) < 0) {
         printf("[test] error closing file.\n");
         return -1;
      }
   }

   // removes  file '/file1.txt' from the root directory
   fd = my_remove("/file1.txt");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }
   
   // removes  file '/f-1' from root directory
   fd = my_remove("/f-1");
   if (fd < 0) {
      printf("[test] unable to remove file.\n");
      return -1;
   }
   
   
   // copy  file '/f-0' to '/f-10' in the root directory
   fd = my_copy("/f-0", "/f-10");
   if (fd < 0) {
      printf("[test] unable to copy a file.\n");
      return -1;
   }
   
   // append  file '/f-2' to '/f-10' in the root directory
   fd = my_append("/f-10", "/f-2");
   if (fd < 0) {
      printf("[test] unable to append a file.\n");
      return -1;
   }
   
         
   // Dump file system occupied data blocks 
   printf("\n---> disk dumping request before defragmentation <----\n");
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }

   // defragment
   fd = my_defrag();
   if (fd < 0) {
      printf("[test] unable to defragment the FS.\n");
      return -1;
   }
   
    // Dump file system occupied data blocks 
   printf("\n---> disk dumping request after defragmentation <----\n"); 	
   if(my_diskusage( )<0){
      printf("[test] error dumping occupied data blocks.\n");
      return -1;
   }
  
      printf("\n[test] defragment was successfull if after defragmentation:\n	EITHER:\n	1) There are 6 contiguous blocks: the first 3 blocks are occupied by the files '/f-0' and '/f-10' and the last 3 blocks are occupied by the files '/f-2' and '/f-10'.\n	OR:\n	2) There are 6 contiguous blocks: the first 3 blocks are occupied by the files '/f-0' and '/f-10' and the last 3 blocks are occupied by the file '/f-10'.\n	And there are more 3 contiguous blocks occupied by file '/f-2'.\n\n	(Please check diskusage dumping on the server terminal).\n\n");
   return 0;
}


