/* 
 * Creates a file with 4 blocks and checks if they are on the cache, then creates another file with 10 blocks; checks if the initial ones aren't on the cache, and then reads the first file created and checks if the 4 blocks are again in the cache
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <myfs.h>

int main(int argc, char **argv)
{
   int fd, num;

   // mandatory to init the myfs layer
   my_init_lib();

   // create the empty file 'cache2.txt' in the root directory
   fd = my_open("/cache2.txt",O_CREATE);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // fill in a buffer with ascii chars
   char data[2*1024];
   for (int i = 0; i < sizeof(data); data[i]='a'+i%10, i++);

   // write the data to the file
   num = my_write(fd, data, sizeof(data));
   if (num < sizeof(data)) {
      printf("[test] error writing data to file.\n");
      return -1;
   }
   
   // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }

  // Dump server's cache of blocks 
  if(my_dumpcache()<0){
      printf("[test] error server's cache dumping.\n");
      return -1;
   }
      
   printf("\n[test] Please check on server terminal that:\n		- blocks 11,12,13 and 14 ARE in the cache.\n\n (Sleeping 5 seconds...).\n\n");
    	
       sleep(5);	
  	
   // create the empty file 'cache2.txt' in the root directory
   fd = my_open("/cache2.1.txt",O_CREATE);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // fill in a buffer with ascii chars
   char data2[5*1024];
   for (int i = 0; i < sizeof(data2); data2[i]='a'+i%10, i++);

   // write the data to the file
   num = my_write(fd, data2, sizeof(data2));
   if (num < sizeof(data2)) {
      printf("[test] error writing data to file.\n");
      return -1;
   }
   
   // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }
    
  // Dump server's cache of blocks 
  if(my_dumpcache()<0){
      printf("[test] error server's cache dumping.\n");
      return -1;
   }
      
   printf("\n[test] Please check on server terminal if:\n		- blocks 11,12,13 and 14 ARE NOT in the cache.\n\n	(Sleeping 5 seconds...).\n\n");
    	
  //getchar();
    sleep(5);
    
   /*
    * check that file 'cache1.txt' is not corrupt
    */
   
   fd = my_open("/cache2.txt",0);
   if (fd < 0) {
      printf("[test] unable to open file.\n");
      return -1;
   }

   // read and compare file contents with written data
   char buffer2[512];
   int read = 0;
   while (1) {
      num = my_read(fd,buffer2,sizeof(buffer2));
      if (num < 0) {
         printf("[test] error reading data from file.\n");
         return -1;
      }

      if (num == 0) {
         break;
      }
      read += num;
   }
   
    // close the file
   if (my_close(fd) < 0) {
      printf("[test] error closing file.\n");
      return -1;
   }
  
  // Dump server's cache of blocks 
  if(my_dumpcache()<0){
      printf("[test] error server's cache dumping.\n");
      return -1;
   }
      
   printf("\n[test] Please check on server terminal if:\n		- blocks 11,12,13 and 14 ARE in the cache again.\n\n");
    	
   return 0;
}


